/* SQL Database Interface. MySql interface.
   Author: adv (Andriy Marchenko), 06 oct, 2001
           <adv@adv.org.ua>  */

#include <mysql/mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "sdbi.h"


__int16_t
sql_mysql_connect(sql_t* drv)
{
  if (!(drv->connection = mysql_init(NULL)))
      return 0;

  if (mysql_real_connect((MYSQL*)drv->connection,    sql_get_opt(drv, "host"), 
                         sql_get_opt(drv, "user"),   sql_get_opt(drv, "password"), 
                         sql_get_opt(drv, "dbname"), sql_get_iopt(drv, "port"),
                         sql_get_opt(drv, "socket"), sql_get_iopt(drv, "flag")) == NULL)
    {
      mysql_close((MYSQL*)drv->connection);
      drv->connection = NULL;

      return 0;
    }

  return 1;
}


void
sql_mysql_disconnect(sql_t* drv)
{
  if (!drv->connection)
      return;

  mysql_close((MYSQL*)drv->connection);
  drv->connection = NULL;
}


void
sql_mysql_freeres(sql_res_t* res)
{
  if (res->result != NULL)
    {
      mysql_free_result((MYSQL_RES*)res->result);
      res->result = NULL;
    }
}


// TODO rewrite
sql_res_t*
sql_mysql_execute(sql_res_t* res, const char* query)
{
  if (mysql_query((MYSQL*)res->drv->connection, query) != 0)
      return res;

  res->result = mysql_store_result((MYSQL*)res->drv->connection);
  if (res->result != NULL)
    {
      res->num_cols = mysql_num_fields((MYSQL_RES*)res->result);
      res->num_rows = mysql_num_rows((MYSQL_RES*)res->result);
      res->field    = mysql_fetch_fields((MYSQL_RES*)res->result);
    }

  return res;
}


__int16_t
sql_mysql_fetch(sql_res_t* res)
{
  res->row = (void**)mysql_fetch_row((MYSQL_RES*)res->result);

  if (res->row == NULL)
    {
//      TODO Fix me
//      sql_mysql_free_result(drv, cursor);

      return 0;
    }

  return 1;
}


const char*
sql_mysql_getfield_pos(sql_res_t* res, __uint16_t field)
{
  return (const char*)res->row[field];
}


const char*
sql_mysql_getfield(sql_res_t* res, const char* field)
{
  __uint32_t i;

  for (i = 0; i < res->num_cols; i++)
    {
      if (strcmp((const char*)((MYSQL_FIELD*)res->field)[i].name, field) == 0)
          return (const char*)res->row[i];
    }

  return NULL;
}


__uint32_t
sql_mysql_fieldlen(sql_res_t* res, __uint16_t field)
{
  return ((__uint32_t*)mysql_fetch_lengths((MYSQL_RES*)res->result))[field];
}


__uint32_t
sql_mysql_lastrec(sql_t* drv)
{
  return mysql_insert_id((MYSQL*)drv->connection);
}


__uint32_t
sql_mysql_numrows(sql_res_t* res)
{
  return res->num_rows;
}


__uint16_t
sql_mysql_numcols(sql_res_t* res)
{
  return res->num_cols;
}


const char*
sql_mysql_fieldname(sql_res_t* res, __uint16_t field)
{
  MYSQL_FIELD* fld = (MYSQL_FIELD*)res->field;
  return fld[field].name;
}
