/* SQL Database Interface. PostgreSQL interface.
   Author: adv (Andriy Marchenko), 06 oct, 2001
           <adv@adv.org.ua>  */

#include <pgsql/libpq-fe.h>
#include <string.h>

#include "types.h"
#include "sdbi.h"


__int16_t
sql_postgres_connect(sql_t* drv)
{
  const char* host       = sql_get_opt(drv, "host");
  const char* hostaddr   = sql_get_opt(drv, "hostaddr");
  const char* port       = sql_get_opt(drv, "port");
  const char* options    = sql_get_opt(drv, "options");
  const char* tty        = sql_get_opt(drv, "tty");
  const char* dbname     = sql_get_opt(drv, "dbname");
  const char* user       = sql_get_opt(drv, "user");
  const char* password   = sql_get_opt(drv, "password");
//  const char* requiressl = sql_get_opt(drv, "requiressl");

  char connect_str[256];

  snprintf(connect_str, sizeof(connect_str),
           " host='%s' hostaddr='%s' port='%s' options='%s' tty='%s'"
           " dbname='%s' user='%s' password='%s'",
           host     ? host     : "",
           hostaddr ? hostaddr : "", 
           port     ? port     : "",
           options  ? options  : "",
           tty      ? tty      : "",
           dbname   ? dbname   : "",
           user     ? user     : "",
           password ? password : "");

  drv->connection = PQconnectdb(connect_str);
//  drv->connection = PQsetdbLogin(host, port, options, tty, database, user, password);

  if (!drv->connection)
      return -1;

  if (PQstatus((PGconn*)drv->connection) == CONNECTION_BAD)
    {
      printf("ERR: %s\n", PQerrorMessage(drv->connection));
      PQfinish((PGconn*)drv->connection);
      return -1;
    }

  return 0;
}


void
sql_postgres_disconnect(sql_t* drv)
{
  PQfinish((PGconn*)drv->connection);
  drv->connection = NULL;
}


void
sql_postgres_freeres(sql_res_t* res)
{
}


sql_res_t*
sql_postgres_execute(sql_res_t* res, const char* query)
{
  res->result = PQexec((PGconn*)res->drv->connection, query);

  res->current_row = -1;
  res->drv->res_last = res;

  return res;
}


__int16_t
sql_postgres_fetch(sql_res_t* res)
{
  res->current_row++;

  if (res->current_row >= PQntuples((PGresult*)res->result))
      return 0;

  return 1;
}


const char*
sql_postgres_getfield_pos(sql_res_t* res, __uint16_t field)
{
  return PQgetvalue((PGresult*)res->result, res->current_row, field);
}


const char*
sql_postgres_getfield(sql_res_t* res, const char* field)
{
  return NULL;
}


__int32_t
sql_postgres_fieldlen(sql_res_t* res, __uint16_t field)
{
  return PQfsize((PGresult*)res->result, field);
}


// TODO
__uint32_t
sql_postgres_lastrec(sql_t* drv)
{
  if (!drv->connection)
      return 0;

  return 0;
}


__uint32_t
sql_postgres_numrows(sql_res_t* res)
{
  return PQntuples((PGresult*)res->result);
}


__uint16_t
sql_postgres_numcols(sql_res_t* res)
{
  return PQnfields((PGresult*)res->result);
}


const char*
sql_postgres_fieldname(sql_res_t* res, __uint16_t field)
{
  return PQfname((PGresult*)res->result, field);
}
